class Messages:
    USER_PROVIDED_CONFIG_HEADER = "All user provided configs for brokers in the cluster are:"


class Errors:
    CONFIG_FILE_SHOULD_BE_PROVIDED = "A configuration file should be provided for connect cluster"
