from kfk.commands.main import kfk
from kfk.commands.clusters import clusters
from kfk.commands.topics import topics
from kfk.commands.users import users
from kfk.commands.configs import configs
from kfk.commands.version import version
from kfk.commands.acls import acls
from kfk.commands.operator import operator
from kfk.commands.connect import connect
from kfk.commands.env import env
from kfk.setup import setup

setup()