SPACE = " "
COLON = ":"
SEMICOLON = ";"
AMPERSAND = "&"
NEW_LINE = "\n"
KAFKA_PORT = "9092"
KAFKA_SECURE_PORT = "9093"
KAFKA_SSL = "SSL"
COMMON_NAME_PREFIX = "CN="
BROKER_TMP_FOLDER_PATH = "/tmp"


class SpecialTexts:
    BROKER_CONFIG_FILE_USER_CONFIG_HEADER = "\n##########\n# User provided configuration\n##########\n"
    CONNECT_BOOTSTRAP_SERVERS = "bootstrap.servers"
    CONNECT_OUTPUT_IMAGE = "output.image"
    CONNECT_PLUGIN_URL = "plugin.url"
    CONNECT_PLUGIN_PATH = "plugin.path"
